//
// $Id: Copying.h,v 1.9 2005/07/12 19:15:36 will_mason Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */

#if !defined(COPYING_OL_GUARD)
#define COPYING_OL_GUARD

#import <ObjectiveLib/Memory.h>

/**
 * @protocol OLCopying Copying.h ObjectiveLib/Copying.h
 *
 * A protocol for ensuring that objects can be copied using ObjectiveLib
 * allocators.
 *
 * @sa OLAllocator
 */
@protocol OLCopying <NSCopying>

/**
 * Copy this object using the given allocator. No attempt should be made to
 * autorelease the returned object, as the sender is expected to own the
 * allocated memory.
 *
 * @param alloc the allocator to use for memory allocation
 * @return a newly forged object that is a copy of this one
 */
- (id) copyWithAllocator: (OLAllocator*)alloc;

/**
 * Copy this object using the given zone. No attempt should be made to
 * autorelease the returned object, as the sender is expected to own the
 * allocated memory.
 *
 * @param zone the zone to use for memory allocation
 * @return a newly forged object that is a copy of this one
 */
- (id) copyWithZone: (NSZone*)zone;

@end

#endif
