//
// $Id: ObjectReplaceTable.m,v 1.3 2005/05/18 06:46:53 will_mason Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */

#import "ObjectReplaceTable.h"
#import "Macros.h"
#import <string.h>

@implementation OLObjectReplaceTable

- (id) initWithAllocator: (OLAllocator*)alloc
{
    [super init];
    allocator = OBJ_RETAIN(alloc);
    refs = [[allocator allocateObject: [OLPointerRefTable class]]
        initWithAllocator: allocator];
    repCapacity = [refs capacity];
    reps = [allocator allocateBytes: repCapacity * sizeof(id)];
    return self;
}

- (void) dealloc
{
    [allocator deallocateObject: refs];
    [allocator deallocateBytes: reps];
    OBJ_RELEASE(allocator);
    [super dealloc];
}

- (void) assign: (id)object replacement: (id)rep
{
    uint32_t index = [refs lookUp: object];

    while (index >= repCapacity)
        [self grow];
    reps[index] = rep;
}

- (void) grow
{
    unsigned newCapacity = (repCapacity << 1) + 1;
    id* newReps = [allocator allocateBytes: newCapacity * sizeof(id)];

    memcpy(newReps, reps, repCapacity * sizeof(id));
    repCapacity = newCapacity;
    [allocator deallocateBytes: reps];
    reps = newReps;
}

- (id) lookUp: (id)object
{
    uint32_t index = [refs lookUp: object];
    return (index == UINT32_MAX) ? object : reps[index];
}

@end
