//
// $Id: UnitTest.h,v 1.6 2005/05/18 06:46:56 will_mason Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */


#import <stdio.h>
#import <Foundation/NSArray.h>

@interface UnitTest : NSObject
{
@private
    FILE*           outStream;
    FILE*           errStream;
    int             errorCount;
    NSMutableArray* tests;
@protected
    NSMutableArray* failedTests;

}

- (id) init;
- (id) initWithOut: (FILE*)o err: (FILE*)e;
- (void) dealloc;

- (void) errInFile: (const char*)file line: (int)line format: (const char* const)format, ...;
- (FILE*) errStream;
- (int) errorCount;
- (BOOL) hasTestWithName: (NSString*)test;
- (void) listFailedTestsToStream: (FILE*)stream delimiter: (char)delim;
- (id) lookUpTestWithName: (NSString*)name;
- (void) logMessage: (const char* const)format, ...;
- (BOOL) preventAutoLoad;
- (FILE*) outStream;
- (void) runAllTests;
- (void) runTest: (id)test;
- (BOOL) runTestWithName: (NSString*)name;
- (void) setStreamOut: (FILE*)o err: (FILE*)e;
- (BOOL) setUp;
- (BOOL) tearDown;
- (int) testCount;
- (NSArray*) testListQualified: (BOOL)flag;

@end

